# -*- Mode: Python; tab-width: 4 -*-
#
#	$Id: Globals.py,v 1.2 1999/06/14 22:39:11 pavlos Exp $
#	Author: Pavlos Christoforou <pavlos@gaaros.com>
#
# ======================================================================
#
# This software, its author and the author's donkey are all certified
# as open source. Use and abuse us in any way you see fit.
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY GAAROS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GAAROS OR HIS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
########################################################################

__version__='$Revision: 1.2 $'[11:-2]

'''Global objects'''

import pdfutilities
import string

OID=pdfutilities.Counter()


## Error classes

class Error:
    'Base Error class'
    pass

class PropertyError(Error):
    '''Reports missing properties'''
    def __init__(self,missing_property):
        self.missing_property=missing_property
        
    def __repr__(self):
        return '<Missing Property: %s>'%self.missing_property
    
class FontError(Error):
    '''Reports missing Font'''
    def __init__(self,missing_font):
        self.missing_font=missing_font

    def __repr__(self):
        return '<Missing Font: %s>'%self.missing_font



## Font related code

standard_fonts=[ 'Courier', 'Courier-Bold', 'Courier-Oblique',
        'Courier-BoldOblique', 'Helvetica', 'Helvetica-Bold',
        'Helvetica-Oblique', 'Helvetica-BoldOblique', 'Times-Roman',
        'Times-Bold', 'Times-Italic', 'Times-BoldItalic',
        'Symbol','ZapfDingbats']



## We need to keep track of used fonts in a resources dictionary.

## Can it be done cleaner??

key_no=pdfutilities.Counter()
resources_map={}
font_map={}
font_list=[]

class Font:
    def __init__(self,font_name):
        font_name=string.strip(font_name)
        if font_name not in standard_fonts:
            raise FontError(font_name)
        else:
            self.OID=OID.incr()
            self.font_name=font_name
            self.key_name='/F%i'%(key_no.incr())

            ## add stuff in some global structures
            font_map[font_name]=self.key_name
            font_list.append(self)
            resources_map[self.key_name]='%i 0 R'%self.OID
            
    def to_PDF(self):
        out=['%i 0 obj'%self.OID,
             '<<',
             '/Type /Font',
             '/Subtype /Type1',
             '/Name %s'%self.key_name,
             '/BaseFont /%s'%self.font_name,
             '/Encoding /WinAnsiEncoding',
             '>>',
             'endobj']
        return string.join(out,'\n')
