# -*- Mode: Python; tab-width: 4 -*-
#
#	$Id: html2pdf.py,v 1.2 1999/06/17 17:42:17 pavlos Exp $
#	Author: Pavlos Christoforou <pavlos@gaaros.com>
#
# ======================================================================
#
# This software, its author and the author's donkey are all certified
# as open source. Use and abuse us in any way you see fit.
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY GAAROS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL GAAROS OR HIS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
########################################################################

__version__='$Revision: 1.2 $'[11:-2]


__doc__="""

HTML to PDF translator.

  Basic HTML to PDF translator. Supports only a limited functionality
  and is intented to provide PDF support to structured-text
  (www.zope.org). The following elements are supported:

  H1,H2,H3,H4,H5,H6,UL,OL,LI,P,EM,STRONG,HR,PRE
  
  HTML text must be well structured. No loose text is allowed, it must
  be eclosed by a <P> </P> pair. This holds for text inside <LI>
  elements as well. BR does not add empty lines. It works similarly to
  TEX's \linebreak and therefore is allowed only inside paragraphs

  Based on httmllib from the standard python distribution.
"""

## Note: BR not implemented yet


from sgmllib import SGMLParser
import elements
from string import strip

class HTMLParser(SGMLParser):
    """HTMLParser receives an empty list as an argument and populates
    it with elements."""
    from htmlentitydefs import entitydefs

    def __init__(self, result,verbose=0):
        SGMLParser.__init__(self, verbose)
        self.text=''
        self.elements=[result]
        self.cel=elements.P() ## current element
        self.start_p_flag=0
        
    def handle_data(self, data):
        self.text='%s%s'%(self.text,data)

    # --------- Top level elememts

    def start_html(self, attrs): pass
    def end_html(self): pass

    def start_head(self, attrs): pass
    def end_head(self): pass

    def start_body(self, attrs): pass
    def end_body(self): pass

    # ------ Head elements

    def start_title(self, attrs):
        pass

    def end_title(self):
        pass

    # ------ Body elements

    # --- Headings

    def start_h1(self, attrs):
        self.cel=elements.H1() ##current element

    def end_h1(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''

    def start_h2(self, attrs):
        self.text=''
        self.cel=elements.H2() ##current element

    def end_h2(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''
        
    def start_h3(self, attrs):
        self.text=''
        self.cel=elements.H3() ##current element

    def end_h3(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''

    def start_h4(self, attrs):
        self.text=''
        self.cel=elements.H4() ##current element

    def end_h4(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''

    def start_h5(self, attrs):
        self.text=''
        self.cel=elements.H5() ##current element

    def end_h5(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''


    def start_h6(self, attrs):
        self.text=''
        self.cel=elements.H6() ##current element

    def end_h6(self):
        self.cel.add_text(self.text)
        self.elements[-1].append(self.cel)
        self.text=''
        
    ## for ZBook compatibility only!!
    start_i=start_h3
    end_i=end_h3

    # --- Block Structuring Elements

    def start_p(self, attrs):
        self.text=''
        self.cel=elements.P()
        
    def end_p(self):
        if strip(self.text):
            self.cel.add_text(self.text)
            self.elements[-1].append(self.cel)
            self.text=''


    def start_pre(self, attrs):
        self.text=''
        self.literal=1
        self.cel=elements.PRE()
        
    def end_pre(self):
        if strip(self.text):
            self.cel.add_text(self.text)
            self.elements[-1].append(self.cel)
            self.text=''

    # --- List Elements

    def start_ul(self, attrs):
        self.text=''
        self.elements[-1].append(elements.UL())
        self.elements.append([])
        
    def end_ul(self):
        if self.elements[-1]:
            self.elements[-2][-1].LI(self.elements[-1])
        del self.elements[-1]

    def start_ol(self, attrs):
        self.text=''
        self.elements[-1].append(elements.OL())
        self.elements.append([])
        
    def end_ol(self):
        if self.elements[-1]:
            self.elements[-2][-1].LI(self.elements[-1])
        del self.elements[-1]

    def do_li(self, attrs):
        if self.elements[-1]:
            self.elements[-2][-1].LI(self.elements[-1])
            self.elements[-1]=[]

    # --- Phrase Markup

    # Idiomatic Elements

    ## hackish

    def start_em(self, attrs):
        if self.text:
            self.cel.add_text(self.text)
            self.text=''

    def end_em(self):
        if self.text:
            self.cel.add_text(self.text,'EM')
            self.text=''
            
    def end_strong(self):
        if self.text:
            self.cel.add_text(self.text,'STRONG')
            self.text=''

    def end_code(self):
        if self.text:
            self.cel.add_text(self.text,'CODE')
            self.text=''
            
    start_strong=start_code=start_a=start_em
    end_a=end_em

    # --- Line Break

    ##def do_br(self, attrs):
        ##self.cel.add_phrases([elements.BR()])

    # --- Horizontal Rule

    def do_hr(self, attrs):
        self.text=''
        self.elements[-1].append(elements.HR())

        
    def unknown_starttag(self, tag, attrs):
        pass

    def unknown_endtag(self, tag):
        pass


def test(args = None):
    import sys,pdflib,styles

    if not args:
        args = sys.argv[1:]

    if args:
        file = args[0]
    else:
        file = 'test.html'
    try:
        f = open(file, 'r')
    except IOError, msg:
        print file, ":", msg
        sys.exit(1)

    data = f.read()

    f.close()

    out=[]
    p = HTMLParser(out,1)
    p.feed(data)
    p.close()

    doc=pdflib.Document()
    doc.initialize()
    doc.set_style(styles.zpdf_doc)
    
    lt=elements.HeaderLine()
    lt['position']='T'

    lb=elements.HeaderLine()
    lb['position']='B'

    htr=elements.Header()
    htr['position']='TR'

    hbr=elements.Header()
    hbr['Header_text']='Zpdf v0.1'
    hbr.add_page_no()
    hbr['position']='BR'

    doc.add_commonElements([lt,lb,hbr,htr])
    doc.add_elements(out)
    doc.format()
    print doc.to_PDF()

if __name__ == '__main__':
    test()


